﻿using BPA.KitChen.GroupMealOrder.Application.BaseDto;
using BPA.KitChen.GroupMealOrder.Core.Entity;
using BPA.KitChen.GroupMealOrder.Core;
using Furion.DependencyInjection;
using Furion.FriendlyException;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;
using BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.Service.AppletOrder.Dtos;

namespace BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.Service.AppletOrder.Services
{
    public class AppletOrderService : IAppletOrderService, ITransient
    {
        /// <summary>
        /// 核心对象：拥有完整的SqlSugar全部功能
        /// </summary>
        private readonly ISqlSugarClient db;
        /// <summary>
        /// construct
        /// </summary>
        /// <param name="sqlSugarRepository"></param>
        public AppletOrderService(ISqlSugarClient _db)
        {
            db = _db;
        }

        public async Task<bool> AddOrder(OrderInsertDto dto)
        {
            try
            {
                db.Ado.BeginTran();
                var check = db.Queryable<BPA_AppletOrder>().Where(x => x.TradeNo == dto.TradeNo).Any();
                if (check) throw Oops.Oh("订单已存在");
                BPA_AppletOrder _order = new();
                _order.Id = Guid.NewGuid().ToString();
                _order.StoreId = dto.StoreId;
                _order.TradeNo = dto.TradeNo;
                _order.TotalAmount = dto.TotalAmount;
                _order.OrderCount = dto.OrderCount;
                _order.Subject = dto.Subject;
                _order.OrderNumber = DateTime.Now.ToString("yyyMMddhhmmss");
                _order.CreateAt = DateTime.Now;
                _order.GroupId = CurrentUser.TenantId;
                List<BPA_AppletSubOrder> SubOrdeList = new();
                List<BPA_AppletOrderGoodsArrt> OrderGoodsArrtList = new();
                List<BPA_AppletOrderGoodsTechnology> OrderGoodsTechnologyList = new();
                if (dto.Orderdetail != null)
                {
                    for (int i = 0; i < dto.Orderdetail.Count; i++)
                    {
                        BPA_AppletSubOrder _subOrder = new();
                        _subOrder.Id = Guid.NewGuid().ToString();
                        _subOrder.OrderId = _order.Id;
                        _subOrder.GoodsNumber = dto.Orderdetail[i].GoodsNumber;
                        _subOrder.GoodsId = dto.Orderdetail[i].GoodsId;
                        _subOrder.GoodsName = dto.Orderdetail[i].GoodsName;
                        _subOrder.GoodsId = dto.Orderdetail[i].GoodsId;
                        _subOrder.isCustom = dto.Orderdetail[i].isCustom;
                        _subOrder.CreateAt = DateTime.Now;
                        SubOrdeList.Add(_subOrder);
                        for (int t = 0; t < dto.Orderdetail[i].GoodsAttr.Count(); t++)
                        {
                            BPA_AppletOrderGoodsArrt goodsAttr = new();
                            goodsAttr.Id = Guid.NewGuid().ToString();
                            goodsAttr.SuborderId = _subOrder.Id;
                            goodsAttr.GoodsId = dto.Orderdetail[i].GoodsAttr[t].GoodsId;
                            goodsAttr.AttributeId = dto.Orderdetail[i].GoodsAttr[t].AttributeId;
                            goodsAttr.AttributeName = dto.Orderdetail[i].GoodsAttr[t].AttributeName;
                            goodsAttr.Count = dto.Orderdetail[i].GoodsAttr[t].Count;
                            goodsAttr.Price = dto.Orderdetail[i].GoodsAttr[t].Price;
                            goodsAttr.CreateAt = DateTime.Now;
                            OrderGoodsArrtList.Add(goodsAttr);
                        }
                        for (int y = 0; y < dto.Orderdetail[i].FormulaDetail.Count; y++)
                        {
                            BPA_AppletOrderGoodsTechnology orderGoodsTechnology = new();
                            orderGoodsTechnology.Id = Guid.NewGuid().ToString();
                            orderGoodsTechnology.SuborderId = _subOrder.Id;
                            orderGoodsTechnology.DeviceId = dto.Orderdetail[i].FormulaDetail[y].DeviceId;
                            orderGoodsTechnology.GoodsId = dto.Orderdetail[i].FormulaDetail[y].GoodsId;
                            orderGoodsTechnology.TechnologyId = dto.Orderdetail[i].FormulaDetail[y].TechnologyId;
                            orderGoodsTechnology.ActionName = dto.Orderdetail[i].FormulaDetail[y].ActionName;
                            orderGoodsTechnology.ActionValue = dto.Orderdetail[i].FormulaDetail[y].ActionValue;
                            orderGoodsTechnology.Sort = dto.Orderdetail[i].FormulaDetail[y].Sort;
                            orderGoodsTechnology.CreateAt = DateTime.Now;
                            OrderGoodsTechnologyList.Add(orderGoodsTechnology);
                        }
                    }
                }
                await db.Insertable(_order).ExecuteCommandAsync();
                await db.Insertable(SubOrdeList).ExecuteCommandAsync();
                await db.Insertable(OrderGoodsArrtList).ExecuteCommandAsync();
                await db.Insertable(OrderGoodsTechnologyList).ExecuteCommandAsync();
                db.Ado.CommitTran();
                return true;
            }
            catch (Exception ex)
            {
                db.Ado.RollbackTran();
                throw Oops.Oh("创建订单失败，失败信息：" + ex.Message);
            }

        }


        /// <summary>
        /// 查询订单
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<OrderViewDto>> GetOrder(OrderQueryDto dto)
        {
            var list = await db.Queryable<BPA_AppletOrder>()
                .WhereIF(!string.IsNullOrWhiteSpace(dto.TradeNo), x => x.TradeNo == dto.TradeNo)
                .WhereIF(!string.IsNullOrWhiteSpace(dto.OrderNumber), x => x.OrderNumber == dto.OrderNumber)
                .Select(x => new OrderViewDto() { Id = x.Id.SelectAll() })
            .Mapper(x =>
            {
                var de = db.Queryable<BPA_AppletSubOrder>().Where(p => p.OrderId == x.Id)
                    .WhereIF(!string.IsNullOrWhiteSpace(dto.GoodsName), x => x.GoodsName.Contains(dto.GoodsName))
                    .Select(x => new OrderdetaiViewDto() { Id = x.Id.SelectAll() })
                    .Mapper(g =>
                    {
                        var GoodsAttr = db.Queryable<BPA_AppletOrderGoodsArrt>().Where(x => x.SuborderId == g.Id).Select(x => new GoodsAttrViewDto() { Id = x.Id.SelectAll() }).ToList();
                        g.GoodsAttr = GoodsAttr;
                        var FormulaDetail = db.Queryable<BPA_AppletOrderGoodsTechnology>().Where(x => x.SuborderId == g.Id).Select(x => new FormulaDetailViewDto() { Id = x.Id.SelectAll() }).ToList();
                        g.FormulaDetail = FormulaDetail;
                    }).ToList();
                x.Orderdetail = de;
            }).ToListAsync();
            return list;
        }


        public async Task<PageUtil> GetOrder(OrderQueryPageDto dto)
        {
            RefAsync<int> total = 0;
            var list = await db.Queryable<BPA_AppletOrder>()
                .WhereIF(!string.IsNullOrWhiteSpace(dto.TradeNo), x => x.TradeNo == dto.TradeNo)
                .WhereIF(!string.IsNullOrWhiteSpace(dto.OrderNumber), x => x.OrderNumber == dto.OrderNumber)
                .WhereIF(!string.IsNullOrWhiteSpace(dto.StoreId), x => x.StoreId == dto.StoreId)
                .WhereIF(!string.IsNullOrWhiteSpace(dto.Subject), x => x.Subject == dto.Subject)
                   .WhereIF(!string.IsNullOrWhiteSpace(dto.CreateAt.ToString()), x => x.CreateAt == dto.CreateAt)
                .Select(x => new OrderViewDto() { Id = x.Id.SelectAll(), CreatedAt = x.CreateAt })
            .Mapper(x =>
            {
                var de = db.Queryable<BPA_AppletSubOrder>().Where(p => p.OrderId == x.Id)
                    .WhereIF(!string.IsNullOrWhiteSpace(dto.GoodsName), x => x.GoodsName.Contains(dto.GoodsName))
                    .Select(x => new OrderdetaiViewDto() { Id = x.Id.SelectAll() })
                    .Mapper(g =>
                    {
                        var GoodsAttr = db.Queryable<BPA_AppletOrderGoodsArrt>().Where(x => x.SuborderId == g.Id).Select(x => new GoodsAttrViewDto() { Id = x.Id.SelectAll() }).ToList();
                        g.GoodsAttr = GoodsAttr;
                        var FormulaDetail = db.Queryable<BPA_AppletOrderGoodsTechnology>().Where(x => x.SuborderId == g.Id).Select(x => new FormulaDetailViewDto() { Id = x.Id.SelectAll() }).ToList();
                        g.FormulaDetail = FormulaDetail;
                    }).ToList();
                x.Orderdetail = de;
            }).ToPageListAsync(dto.Current, dto.PageSize, total);
            PageUtil util = new PageUtil()
            {
                Total = total,
                Data = list
            };
            return util;
        }
    }
}
