﻿using BPA.KitChen.GroupMealOrder.Application.BaseDto;
using BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.BaseDto;
using BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.Service.WeChatOrder.Dtos;
using BPA.KitChen.GroupMealOrder.Application.Service.Order.Dtos;
using BPA.KitChen.GroupMealOrder.Application.Service.WeighOrder.Dtos;
using BPA.KitChen.GroupMealOrder.Core;
using BPA.KitChen.GroupMealOrder.Core.Common;
using BPA.KitChen.GroupMealOrder.Core.Entity;
using BPA.KitChen.GroupMealOrder.Core.Enum;
using BPA.KitChen.GroupMealOrder.SqlSugar;
using Furion;
using Furion.DatabaseAccessor;
using Furion.DependencyInjection;
using Furion.FriendlyException;
using Microsoft.EntityFrameworkCore;
using Newtonsoft.Json;
using StackExchange.Profiling.Internal;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.Service.WeChatOrder.Service
{
    public class WeChatOrderService : IWeChatOrderService, ITransient
    {
        private SnowFlakeHelper snowFlakeHelper = new SnowFlakeHelper(1, 1);

        /// <summary>
        /// 下单
        /// </summary>
        /// <param name="input"></param>
        /// <param name="AppId"></param>
        /// <param name="GroupId"></param>
        /// <param name="Version"></param>
        /// <returns></returns>
        public async Task<string> CreateOrder(CreateOrderInputDto inputDto)
        {
            var order = new BPA_Order()
            {
                EatType = 0,
                AfterStatus = 0,
                CreatedAt = DateTime.Now,
                CreatedBy = "",
                CusmetMoney = 0,
                CustomerId = inputDto.CustomerId,
                DiscountMoney = 0,
                GroupId = CurrentUser.TenantId,
                Id = Guid.NewGuid().ToString(),
                OrderCount = inputDto.GoodsInfos.Sum(x => x.Count),
                OrderNumber = snowFlakeHelper.NextId().ToString(),
                OrderOriginalMoney = 0,
                OrderRealMoney = 0,
                OrderSettlementStatus = 0,
                OrderSourceFrom = "0",
                OrderStatus = 0,
                OrderType = 0,
                OrgId = CurrentUser.StoreId,
                PayMode = 0,
            };

            var subInfoList = new List<BPA_SubOrder>();
            var goodsList = await GetGoodsList(inputDto.GoodsInfos.Select(x => x.GoodId).ToList());
            if (goodsList == null)
            {
                throw Oops.Oh("商品不存在");
            }
            foreach (var goods in inputDto.GoodsInfos)
            {
                var thisgoods = goodsList.Where(x => x.Id == goods.GoodId).FirstOrDefault();
                subInfoList.Add(new BPA_SubOrder()
                {
                    CookingStatus = 1,
                    CreatedAt = DateTime.Now.ToString(),
                    CreatedBy = "",
                    GoodId = goods.GoodId,
                    GoodsRealPrice = thisgoods.Price,
                    Id = Guid.NewGuid().ToString(),
                    IsDevice = 1,
                    IsVaild = 0,
                    OrderId = order.Id,
                });
            }

            var res =await SqlSugarDb.Db.Insertable(order).ExecuteCommandAsync();
            if (res>0)
            {
                await SqlSugarDb.Db.Insertable(subInfoList).ExecuteCommandAsync();
            }

            return order.Id;
        }



        /// <summary>
        /// 获取商品信息
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        private async Task<List<GoodsInfoBaseView>> GetGoodsList(List<string> ids)
        {
            var res = new List<GoodsInfoBaseView>();

            try
            {
                Dictionary<string, string> dic = new Dictionary<string, string>();
                dic.Add("GroupId", CurrentUser.TenantId);
                var url = App.Configuration["GroupMealBaseUrl"] + "api/goods/GetGoodsListByIds";
                var inputData = JsonConvert.SerializeObject(ids);
                var jsonData = HttpHelper.PostData(url, inputData, Encoding.UTF8, "application/json", dic);
                var data = JsonConvert.DeserializeObject<ResultSAASManageDto<List<GoodsInfoBaseView>>>(jsonData);

                if (data.statusCode != 200)
                {
                    throw Oops.Oh(data.errors);
                }

                return data.data;
            }
            catch (Exception e)
            {
                throw Oops.Oh("请求错误");
            }

            return res;
        }


    }
}
