﻿using Essensoft.Paylink.WeChatPay.V2.Notify;
using Essensoft.Paylink.WeChatPay.V2;
using Essensoft.Paylink.WeChatPay;
using Furion.DependencyInjection;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Extensions.Options;
using Org.BouncyCastle.Asn1.Ocsp;
using Furion;
using BPA.KitChen.GroupMealOrder.Core.Entity;

namespace BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.Service.WeighOrderPay.Service
{
    public class WeighOrderPayNotify: IWeighOrderPayNotify, ITransient
    {
        private readonly IWeChatPayNotifyClient _client;
        private readonly IOptions<WeChatPayOptions> _optionsAccessor;

        public WeighOrderPayNotify( IWeChatPayNotifyClient client, 
            IOptions<WeChatPayOptions> optionsAccessor)
        {
            _client = client;
            _optionsAccessor = optionsAccessor;
        }

        /// <summary>
        /// 统一下单支付结果通知
        /// </summary>
        public async Task<IActionResult> Unifiedorder()
        {
            try
            {
               var data= App.HttpContext.Request;
                var notify = await _client.ExecuteAsync<WeChatPayUnifiedOrderNotify>(data, _optionsAccessor.Value);

                //查询订单
                var order = await SqlSugar.SqlSugarDb.Db.Queryable<BPA_WeighOrder>()
                    .ClearFilter()
                    .FirstAsync(x => x.Id == notify.OutTradeNo);
                if (order == null)
                {
                    return WeChatPayNotifyResult.Failure;
                }

                if (notify.ReturnCode == WeChatPayCode.Success)
                {
                    order.States = 1;
                    order.PayStates = 1;

                    await SqlSugar.SqlSugarDb.Db.Updateable(order).ExecuteCommandAsync();

                    return WeChatPayNotifyResult.Success;
                }

                return WeChatPayNotifyResult.Failure;
            }
            catch (WeChatPayException ex)
            {
                return WeChatPayNotifyResult.Failure;
            }
        }

        /// <summary>
        /// 退款结果通知
        /// </summary>
        public async Task<IActionResult> Refund()
        {
            try
            {
                var data = App.HttpContext.Request;

                var notify = await _client.ExecuteAsync<WeChatPayRefundNotify>(data, _optionsAccessor.Value);

                if (notify.ReturnCode == WeChatPayCode.Success)
                {
                    if (notify.RefundStatus == WeChatPayRefundStatus.Success)
                    {
                       
                        return WeChatPayNotifyResult.Success;
                    }
                }

                return WeChatPayNotifyResult.Failure;
            }
            catch (WeChatPayException ex)
            {
                return WeChatPayNotifyResult.Failure;
            }
        }
    }
}
