﻿using BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.Service.WeighOrderPay.Dtos;
using BPA.KitChen.GroupMealOrder.Core.Entity;
using Essensoft.Paylink.WeChatPay.V2;
using Essensoft.Paylink.WeChatPay;
using Furion.DependencyInjection;
using Furion.FriendlyException;
using Microsoft.Extensions.Options;
using System.Net;
using Essensoft.Paylink.WeChatPay.V2.Request;
using System.Text.Json;
using Furion;

namespace BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.Service.WeighOrderPay.Service
{
    public class WeighOrderPayService : IWeighOrderPayService, ITransient
    {
        private readonly IWeChatPayClient _client;
        private readonly IOptions<WeChatPayOptions> _optionsAccessor;

        public WeighOrderPayService(IWeChatPayClient client, IOptions<WeChatPayOptions> optionsAccessor)
        {
            _client = client;
            _optionsAccessor = optionsAccessor;
        }


        /// <summary>
        /// 订单支付
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<WeChatPayResponseDto> PayOrder(PayWeighOrderDto inputDto)
        {
            var result = new WeChatPayResponseDto();

            var order = await SqlSugar.SqlSugarDb.Db.Queryable<BPA_WeighOrder>()
                .ClearFilter()
                .FirstAsync(x => x.Id == inputDto.OrderId);

            if (order == null)
            {
                throw Oops.Oh("订单不存在");
            }

            if (order.PayStates == 1)
            {
                throw Oops.Oh("订单已支付");
            }

            IPHostEntry ipHost = Dns.GetHostEntry(Dns.GetHostName());//Dns.GetHostEntry方法没有过时，但是获取的是v4和v6的所有ip地址
            IPAddress ipAddress = ipHost.AddressList.Where(x => x.AddressFamily.ToString() == "InterNetwork").FirstOrDefault();//InterNetwork是v4的IP，但是依然是多个

            var request = new WeChatPayUnifiedOrderRequest
            {
                Body = order.Id,
                OutTradeNo = order.Id,
                TotalFee = Convert.ToInt32(order.TotalAmount * 100),
                SpBillCreateIp = ipAddress.ToString(),
                NotifyUrl = App.Configuration["DomainName"] +"api/ExternalPlatform/WeighOrder/Unifiedorder",
                TradeType = "JSAPI",
                OpenId = inputDto.OpenId
            };

            var response = await _client.ExecuteAsync(request, _optionsAccessor.Value);
            if (response.ReturnCode == WeChatPayCode.Success && response.ResultCode == WeChatPayCode.Success)
            {
                var req = new WeChatPayMiniProgramSdkRequest
                {
                    Package = "prepay_id=" + response.PrepayId
                };

                var parameter = await _client.ExecuteAsync(req, _optionsAccessor.Value);

                // 将参数(parameter)给 小程序前端 让他调起支付API(https://pay.weixin.qq.com/wiki/doc/api/wxa/wxa_api.php?chapter=7_7&index=5)
                result.parameter = JsonSerializer.Serialize(parameter);
                result.response = response.Body;
                return result;
            }

            result.response = response.Body;
            return result;
        }
    }
}
