﻿using BPA.KitChen.GroupMealOrder.Application.BaseDto;
using BPA.KitChen.GroupMealOrder.Application.Service.AExternalPlatform.Service.CheckService.Services;
using BPA.KitChen.GroupMealOrder.Core;
using BPA.KitChen.GroupMealOrder.Core.CacheOption;
using BPA.KitChen.GroupMealOrder.Core.Entity;
using BPA.KitChen.GroupMealOrder.SqlSugar;
using Furion;
using Microsoft.AspNetCore.Mvc.Controllers;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.EntityFrameworkCore.Metadata.Internal;
using Newtonsoft.Json;
using NPOI.SS.Formula.Functions;
using SqlSugar;
using System.Reflection;

namespace BPA.KitChen.GroupMealOrder.Handlers
{
    public class RequestAuditFiltercs : IAsyncActionFilter
    {
        private readonly ICheckServices _checkServices;

        public RequestAuditFiltercs( ICheckServices checkServices)
        {
            _checkServices = checkServices;
        }

        public async Task OnActionExecutionAsync(ActionExecutingContext context, ActionExecutionDelegate next)
        {

            CurrentUser.MessageId=Guid.NewGuid().ToString();

            //============== 这里是执行方法之前获取数据 ====================
            // 获取 HttpContext 和 HttpRequest 对象
            var httpContext = context.HttpContext;
            var httpRequest = httpContext.Request;

            // 获取客户端 Ipv4 地址
            var remoteIPv4 = httpContext.GetRemoteIpAddressToIPv4();

            // 获取请求的 Url 地址
            var requestUrl = httpRequest.GetRequestUrlAddress();

            // 获取来源 Url 地址
            var refererUrl = httpRequest.GetRefererUrlAddress();

            // 获取控制器、路由信息
            var actionDescriptor = context.ActionDescriptor as ControllerActionDescriptor;

            // 获取请求的方法
            var method = actionDescriptor.MethodInfo;

            // 获取请求参数（写入日志，需序列化成字符串后存储）
            var parameters = context.ActionArguments;


            //foreach (var parameter in parameters)
            //{
            //    var stingA = DtoValidator.GetSign(parameter.Value);
            //    var vvv = 00;
            //}


            //  配置信息

            var configList = await SqlSugarDb.Db.Queryable<BPA_WeChatConfig, BPA_WeChatConfigDetail>(
                    (a, b) => new JoinQueryInfos(JoinType.Left, a.Key == b.Key))
                .ClearFilter()
                // .Where((a,b)=> a.Key == "wx37fa34ae664500d2")
                .Select((a, b) => new
                {
                    Key = a.Key,
                    AppId = b.AppId,
                    AppMessageActivity = b.AppMessageActivity,
                    AppMessagetTakeAway = b.AppMessagetTakeAway,
                    b.AppSecret,
                    b.FranchiseeAddress,
                    b.IsProduction,
                    b.MchId,
                    b.MchIP,
                    b.MchKey,
                    b.OrderAddress,
                    b.WeChatOrderNotifyUrl,
                    b.WeChatPayNotifyUrl,
                    b.WeChatRefundUrl,
                    b.APIKey,
                    b.APIPrivateKey,
                    b.APIv3Key,
                    b.Certificate,
                    b.CertificatePassword,
                    b.RsaPublicKey,
                    b.SubAppId,
                    b.SubMchId,


                })
               .ToListAsync();

            var list = new List<ApolloApplicationConfig>();

            foreach (var config in configList)
            {
                list.Add(new ApolloApplicationConfig()
                {
                    Key = config.Key,
                    WechatConfig = new WechatConfigOptions()
                    {
                        Appid = config.AppId,
                        AppMessageActivity = config.AppMessageActivity,
                        AppMessagetTakeAway = config.AppMessagetTakeAway,
                        AppSecret = config.AppSecret,
                        FranchiseeAddress = config.FranchiseeAddress,
                        IsProduction =Convert.ToBoolean(config.IsProduction),
                        MchId = config.MchId,
                        MchIP = config.MchIP,
                        MchKey = config.MchKey,
                        OrderAddress = config.OrderAddress,
                        WeChatOrderNotifyUrl = config.WeChatOrderNotifyUrl,
                        WeChatPayNotifyUrl = config.WeChatPayNotifyUrl,
                        WeChatRefundUrl = config.WeChatRefundUrl

                    },
                    WechatPayConfig = new WechatPayConfigOptions()
                    {
                        APIKey = config.APIKey,
                        APIPrivateKey = config.APIPrivateKey,
                        APIv3Key = config.APIv3Key,
                        AppId = config.AppId,
                        AppSecret = config.AppSecret,
                        Certificate = config.Certificate,
                        CertificatePassword = config.CertificatePassword,
                        MchId = config.MchId,
                        RsaPublicKey = config.RsaPublicKey,
                        SubAppId = config.SubAppId,
                        SubMchId = config.SubMchId
                    }

                });
            }

            ApolloApplication.Configs = list;




            var groupId = httpRequest.Headers["GroupId"];
            if (!string.IsNullOrEmpty(groupId))
            {
                CurrentUser.TenantId = groupId;
            }
            if (requestUrl.ToUpper().Contains("ExternalPlatform".ToUpper()))
            {
               

                var key = httpRequest.Headers["key"];
                var sign = httpRequest.Headers["sign"];
             
                List<PropertyInfo> proplist = new List<PropertyInfo>();
                foreach (var parameter in parameters)
                {
                   // var stingA = DtoValidator.GetSign(parameter.Value);
                    //var sign = DtoValidator.GetAttributePrice(parameter.Value, "sign");
                    await _checkServices.CheckSign(key, "stingA", sign);
                }
            }

            //============== 这里是执行方法之后获取数据 ====================
            var actionContext = await next();

            // 获取返回的结果
            var returnResult = actionContext.Result;

            // 判断是否请求成功，没有异常就是请求成功
            var isRequestSucceed = actionContext.Exception == null;
            if (requestUrl.ToUpper().Contains("ExternalPlatform".ToUpper()))
            {
                try
                {
                    //记录日志
                    await SqlSugarDb.DbLog.Insertable(new BPA_ThirdpartyRequestLog()
                    {
                        Id = CurrentUser.MessageId,
                        CreateAt = DateTime.Now,
                        GroupId = CurrentUser.TenantId,
                        RequestUrl = requestUrl,
                        ReauestMethod = httpRequest.Method,
                        ReauestParm = JsonConvert.SerializeObject(parameters),
                        ResultParm = !isRequestSucceed ? actionContext.Exception.ToString() : JsonConvert.SerializeObject(returnResult),
                        IP = remoteIPv4
                    }).ExecuteCommandAsync();

                    ////记录下发记录
                    //if (isRequestSucceed)
                    //{
                    //    foreach (var parameter in parameters)
                    //    {

                    //        var isPush = DtoValidator.GetAttributePrice(parameter.Value, "isPush");
                    //        var callbackUrl = DtoValidator.GetAttributePrice(parameter.Value, "callbackUrl");
                    //        var storeIdList = DtoValidator.GetAttributePrice(parameter.Value, "storeIdList");
                    //        var deviceIdList = DtoValidator.GetAttributePrice(parameter.Value, "deviceIdList");

                    //        if (isPush.ToUpper() == "true".ToUpper())
                    //        {
                    //            var list = new List<string>();
                    //            if (!string.IsNullOrEmpty(deviceIdList))
                    //            {
                    //                list = JsonConvert.DeserializeObject<List<string>>(deviceIdList);
                    //            }
                    //            if ( !string.IsNullOrEmpty(storeIdList))
                    //            {
                    //                list = await _thirdpartyPushService.GetDeviceByStoreId(JsonConvert.DeserializeObject<List<string>>(storeIdList));
                    //            }

                    //            await _thirdpartyPushService.AddThirdpartyPushRecord(new BPA_ThirdpartyPushRecord()
                    //            {
                    //                Id = Guid.NewGuid().ToString(),
                    //                GroupId = CurrentUser.GroupId,
                    //                IsPush = false,
                    //                CallbackUrl = callbackUrl,
                    //                MessageId = msgId,
                    //                PushData = JsonConvert.SerializeObject(parameters),
                    //                PushDevice = JsonConvert.SerializeObject(list)
                    //            });
                    //        }
                    //        break;
                    //    }
                    //}


                }
                catch (Exception e)
                {
                }
            }

        }
    }
}
