﻿using BPA.KitChen.GroupMealOrder.Core.CacheOption;
using BPA.KitChen.WeChat.WechatServer.Dtos;
using Essensoft.Paylink.WeChatPay;
using Essensoft.Paylink.WeChatPay.V2.Response;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using System.Threading.Tasks;

namespace BPA.KitChen.WeChat.WechatServer.Service
{
    /// <summary>
    /// 微信支付服务
    /// </summary>
    public interface IWechatPayService
    {
        /// <summary>
        /// 付款
        /// </summary>
        /// <param name="openid"></param>
        /// <param name="ordernumber"></param>
        /// <returns></returns>
        Task<IActionResult> PayLinkWechatPay(string openid, string ordernumber);
        /// <summary>
        /// 充值
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<IActionResult> RechargePay(RechargeInput input);
        /// <summary>
        /// Native付款,生成二维码
        /// </summary>
        /// <param name="amount">付款金额(分)</param>
        /// <param name="orderid">订单ID</param>
        /// <returns></returns>
        string NativePay(decimal amount, string orderid);
        /// <summary>
        /// 退款
        /// </summary>
        /// <param name="input"></param>
        /// <param name="config"></param>
        /// <returns></returns>
        Task<RefundDto> Refund(WechatRefundInput input, ApolloApplicationConfig config);
        /// <summary>
        /// 查询订单
        /// </summary>
        /// <param name="appId"></param>
        /// <param name="transaction_id"></param>
        /// <param name="out_trade_no"></param>
        /// <returns></returns>
        Task<WeChatPayOrderQueryResponse> PayOrderQuery(string appId,string transaction_id, string out_trade_no);
        /// <summary>
        /// 付款码支付
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<IActionResult> MicropayPay(MicropayInput input);
    }
}
